@@ -56,6 +56,7 @@ modules = [
     "exhentai",
     "facebook",
     "fanbox",
+    "fansly",
     "fantia",
     "fapello",
     "fapachi",
@@ -190,6 +191,7 @@ modules = [
     "tsumino",
     "tumblr",
     "tumblrgallery",
+    "tungsten",
     "twibooru",
     "twitter",
     "urlgalleries",
